/* SPDX-License-Identifier: GPL-2.0-or-later
 * Copyright (c) Linux Test Project, 2022
 */

#ifndef TST_RAND_DATA_H__
#define TST_RAND_DATA_H__

#include <stddef.h>

/* Includes null byte */
extern const size_t tst_rand_data_len;
/* statically defined random data */
extern const char *const tst_rand_data;

#endif
